#include "Precompiled.h"
#include "..\Calculator.wsdl.h"
#include "..\WsHelpers.h"

HRESULT Run(__in PCWSTR url,
            __in double first, 
            __in double second, 
            __out double* result, 
            __in_opt WS_ERROR* error)
{
    ASSERT(0 != url);
    ASSERT(0 != result);

    HRESULT hr = S_OK;

    WsHeap heap;

    HR(heap.Create(100, // max size
                   0, // trim size
                   0, // properties
                   0, // property count
                   error));

    WsServiceProxy serviceProxy;

    HR(CalculatorBinding_CreateServiceProxy(0, // template value
                                            0, // properties
                                            0, // property count
                                            &serviceProxy,
                                            error));

    WS_ENDPOINT_ADDRESS address =
    {
        {
            static_cast<ULONG>(wcslen(url)),
            const_cast<PWSTR>(url)
        }
    };

    HR(serviceProxy.Open(&address,
                         0, // async context
                         error));

    HR(CalculatorBinding_Add(serviceProxy,
                             first,
                             second,
                             result,
                             heap,
                             0, // properties
                             0, // property count
                             0, // async context
                             error));

    HR(serviceProxy.Close(0, // async context
                          error));

    return S_OK;
}

int main()
{
    HRESULT hr = S_OK;

    wprintf(L"Press any key to START the client.");
    _getch();
    wprintf(L"\n");

    WsError error;

    HR(error.Create(0, // properties
                    0)); // property count

    const double first = 1.23;
    const double second = 2.34;
    double result = 0.0;

    hr = Run(L"http://localhost:81/calculator",
             first,
             second,
             &result,
             error);

    if (SUCCEEDED(hr))
    {
        ASSERT(3.57 == result);

        wprintf(L"%f + %f = %f\n",
                first,
                second,
                result);
    }
    else
    {
        ULONG stringCount = 0;

        HR(error.GetProperty(WS_ERROR_PROPERTY_STRING_COUNT,
                             &stringCount));

        for (ULONG i = 0; i < stringCount; ++i)
        {
            WS_STRING string;

            HR(error.GetString(i,
                               &string));

            wprintf(L"Error %d: %.*s\n",
                    i,
                    string.length,
                    string.chars);
        }
    }

    _getch();
}
