#include "Precompiled.h"
#include "..\Calculator.wsdl.h"
#include "..\WsHelpers.h"

HRESULT CALLBACK AddCallback(__in const WS_OPERATION_CONTEXT* /*operationContext*/,
                             __in double first, 
                             __in double second, 
                             __out double* result, 
                             __in_opt const WS_ASYNC_CONTEXT* /*asyncContext*/,
                             __in_opt WS_ERROR* /*error*/)
{
    ASSERT(0 != result);

    *result = first + second;

    return S_OK;
}

HRESULT Run(__in PCWSTR url,
            __in_opt WS_ERROR* error)
{
    ASSERT(0 != url);

    HRESULT hr = S_OK;

    WsHeap heap;

    HR(heap.Create(250, // max size
                   0, // trim size
                   0, // properties
                   0, // property count
                   error));


    const WS_STRING address =
    {
        static_cast<ULONG>(wcslen(url)),
        const_cast<PWSTR>(url)
    };

    CalculatorBindingFunctionTable functions =
    {
        AddCallback
    };

    WS_SERVICE_ENDPOINT* endpoint = 0;

    HR(CalculatorBinding_CreateServiceEndpoint(0, // template value
                                               &address,
                                               &functions,
                                               0, // authz callback
                                               0, // properties
                                               0, // property count
                                               heap,
                                               &endpoint,
                                               error));

    const WS_SERVICE_ENDPOINT* endpoints[] = { endpoint };

    WsServiceHost serviceHost;

    HR(serviceHost.Create(endpoints,
                          _countof(endpoints),
                          0, // properties
                          0, // property count
                          error));

    HR(serviceHost.Open(0, // async context
                        error));

    wprintf(L"Press any key to STOP the server.");
    _getch();
    wprintf(L"\n");

    HR(serviceHost.Close(0, // async context
                         error));

    return S_OK;
}

int main()
{
    HRESULT hr = S_OK;

    WsError error;

    HR(error.Create(0, // properties
                    0)); // property count

    hr = Run(L"http://+:81/calculator",
             error);

    if (FAILED(hr))
    {
        ULONG stringCount = 0;

        HR(error.GetProperty(WS_ERROR_PROPERTY_STRING_COUNT,
                             &stringCount));

        for (ULONG i = 0; i < stringCount; ++i)
        {
            WS_STRING string;

            HR(error.GetString(i,
                               &string));

            wprintf(L"Error %d: %.*s\n",
                    i,
                    string.length,
                    string.chars);
        }

        _getch();
    }
}
